<?php

namespace Drupal\sber_id\Controller;

use Drupal\sber_id\Plugin\Network\SberIdHandler;
use Drupal\user\Entity\User;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Cache\CacheableMetadata;

use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\JsonResponse;
class SberIdController extends ControllerBase {

	static $sber_id_users_table = 'sber_id_users';

	private $moduleSettings = [];
	private $oauth_handler;

	public function Auth() {

		$module_path = DRUPAL_ROOT . '/' . drupal_get_path('module', 'sber_id');
		$log_dir =  $module_path . '/log/';
		$sert_dir =  $module_path . '/sert/';

	    $moduleConfig = \Drupal::config('sber_id.settings');
	    
	    $this->moduleSettings = [
			'api_mode' => $moduleConfig->get('api_mode'),
			'client_id' => $moduleConfig->get('client_id'),
			'client_secret' => $moduleConfig->get('client_secret'),
			'sertificate_password' => $moduleConfig->get('sertificate_password'),
			'test_client_id' => $moduleConfig->get('test_client_id'),
			'test_client_secret' => $moduleConfig->get('test_client_secret'),
			'test_sertificate_password' => $moduleConfig->get('test_sertificate_password'),
			// 'web_to_app' => $moduleConfig->get('use_mweb2app'),
			'pkce' => $moduleConfig->get('use_pkce'),
			'scopes' => $moduleConfig->get('scopes'),
			'button_size' => $moduleConfig->get('button_size'),
			'button_form' => $moduleConfig->get('button_form'),
			'button_style' => $moduleConfig->get('button_style'),
			'fast_login' => $moduleConfig->get('fast_login'),
			'personal_button' => $moduleConfig->get('personal_button'),
	    ];

	    $this->oauth_handler = new SberIdHandler( $moduleConfig->get('api_version'), $moduleConfig->get('api_mode'));

	    // $this->oauth_handler->debugOn();
        // $this->oauth_handler->testModeOn();
        
        
        $this->oauth_handler->setSertificateDir($sert_dir);
        $this->oauth_handler->setLogDir($log_dir);

	    if($moduleConfig->get('api_mode') === 'TEST') {
	      $this->oauth_handler->setClientId($moduleConfig->get('test_client_id'));
	      $this->oauth_handler->setClientSecret($moduleConfig->get('test_client_secret'));
	      $this->oauth_handler->setSertificatePassword($moduleConfig->get('test_sertificate_password'));
	    } else {
	      $this->oauth_handler->setClientId($moduleConfig->get('client_id'));
	      $this->oauth_handler->setClientSecret($moduleConfig->get('client_secret'));
	      $this->oauth_handler->setSertificatePassword($moduleConfig->get('sertificate_password'));
	    }

    	$this->oauth_handler->setProperty('pkce',false);

		if(isset($_GET['continue_auth']) && $_GET['continue_auth'] == 1) {
			$user_data = array(
			    'email' => $_POST['sber_id_user']['email'],
			    'phone' => $_POST['sber_id_user']['phone'],
			    'sub' => $_POST['sber_id_user']['sub'],
			    'aud' => $_POST['sber_id_user']['aud'],
			    'family_name' => $_POST['sber_id_user']['family_name'],
			    'given_name' => $_POST['sber_id_user']['given_name'],
			    'middle_name' => $_POST['sber_id_user']['middle_name'],
			);
			if($_POST['check_email'] == '1') {
				if(!filter_var($_POST['sber_id_user']['email'], FILTER_VALIDATE_EMAIL)) {
					$user_data['email'] = '';
					return $this->show_login_form($user_data, false, true);
				} else {
					$finded_drupal_user = $this->drupal_user_find_by_email($user_data['email']);
					$finded_sberid_user = $this->sber_id_user_find_by_sub($user_data['sub']);

					if($finded_sberid_user && $finded_drupal_user) {
			    		$this->finish_auth($finded_drupal_user);
			    		return $this->redirect('<front>');
			    	}
			    	if($finded_sberid_user) {
			    		$drupal_user = $this->drupal_user_find_by_id($finded_sberid_user->user_id);
			    		if(!$drupal_user) {
							$drupal_user = $this->drupal_user_create($user_data);
			    		}
			    		$this->finish_auth($drupal_user);
			    		return $this->redirect('<front>');
			    	}
			    	if($finded_drupal_user && !$finded_sberid_user) {
			    		return $this->show_login_form($user_data, $finded_drupal_user);
			    	}
				}
			}
			$user_data['username'] = $_POST['username'];
			$user_data['password'] = $_POST['password'];

			$uid = \Drupal::service('user.auth')->authenticate($user_data['username'], $user_data['password']);
			if($uid) {
				$userInfo = $this->drupal_user_find_by_id($uid);
				$finded_sberid_user = $this->sber_id_user_find_by_sub($user_data['sub']);
				if($finded_sberid_user) {
					$new_sber_id_user = $finded_sberid_user;
				} else {
					$new_sber_id_user = $this->sber_id_user_create($user_data, $userInfo);
				}
				if($new_sber_id_user) {
					$this->finish_auth($userInfo);
					if(isset($_GET['fast_login'])) {
						return new JsonResponse([
					     	'status' => 1,
			            	'auth_status' => 1
					    ]);	
					} else {
						return $this->redirect('<front>');
					}
				}
			} else {
				$finded_drupal_user = $this->drupal_user_find_by_email($user_data['email']);
				return $this->show_login_form($user_data, $finded_drupal_user, true);
			}

		}

        if(!isset($_GET['code'])) {
        	return false;
        }

        // $this->oauth_handler->setProperty('code_verifier', isset($_SESSION['sberbankid_code_verifier']) ? $_SESSION['sberbankid_code_verifier'] : false );
      	// unset($_SESSION['sberbankid_code_verifier']);

		$this->oauth_handler->setProperty('code', $_GET['code']);

    	$this->oauth_handler->reqGetAccessToken();
    	$this->oauth_handler->reqGetUserInfo();
    	$sber_user_info = $this->oauth_handler->getUserinfo();
    	if($this->oauth_handler->authSuccess()) {
    		
	    	// if(!isset($sber_user_info['email']) || strlen($sber_user_info['email'] < 4)) {
	    	// 	return $this->show_login_form($sber_user_info, false);
	    	// }
			// echo 123;
			// die;
	    	$finded_sberid_user = $this->sber_id_user_find_by_sub($sber_user_info['sub']);
	    	$finded_drupal_user = $this->drupal_user_find_by_email($sber_user_info['email']);
	    	if($finded_sberid_user && $finded_drupal_user) {
	    		
	    		if(isset($_GET['fast_login'])) {
	    			$this->finish_auth($finded_drupal_user);
					return new JsonResponse([
				     	'status' => 1,
		            	'auth_status' => 1
				    ]);	
				} else {
					$this->finish_auth($finded_drupal_user);
					return $this->redirect('<front>');
				}
	    	}

	    	if($finded_sberid_user) {
	    		$drupal_user = $this->drupal_user_find_by_id($finded_sberid_user->user_id);
	    		if(!$drupal_user) {
					$drupal_user = $this->drupal_user_create($sber_user_info);
	    		}
	    		$this->finish_auth($drupal_user);
	    		return $this->redirect('<front>');
	    	}
	    	if($finded_drupal_user && !$finded_sberid_user) {
	    		return $this->show_login_form($sber_user_info, $finded_drupal_user);
	    	}

	    	$new_drupal_user = $this->drupal_user_create($sber_user_info);
	    	$new_sber_id_user = $this->sber_id_user_create($sber_user_info, $new_drupal_user);
	    	if($new_drupal_user && $new_sber_id_user) {
	    		$this->finish_auth($new_drupal_user);
	    		return $this->redirect('<front>');
	    	}

    	} 

    	$this->oauth_handler->logger();	
    	$this->oauth_handler->showError();
		return new JsonResponse([
	      'auth_url' => 'error',
	      'method' => 'GET',
	    ]);

	}

	// Create new Drupal in user table
	private function sber_id_user_create($user_data,$drupal_user) {
		if(!$user_data) {
			return false;
		}
		$connection = \Drupal::service('database');
		$result = $connection->insert(self::$sber_id_users_table)
		  ->fields([
		    'sub' => $user_data['sub'],
		    'aud' => $user_data['aud'],
		    'linked_account_id' => $drupal_user->id(),
		    'email' => $user_data['email'],
		    'phone' => isset($user_data['phone']) ? $user_data['phone'] : '',
		    'name' => isset($user_data['given_name']) ? $user_data['given_name'] : '',
		    'user_id' => $drupal_user->id(),
		    'created' => \Drupal::time()->getRequestTime(),
		  ])
		  ->execute();

		if($result) {
		 	return $result;
		}
		return false;
	}

	// Create new in Sber_Id user table
 	private function drupal_user_create($user_data) {
 		if(!$user_data) {
			return false;
		}
 		$sberIdLogin = "sberid" . substr($user_data['sub'], 0,6);

 		$userCreation = User::create(array(
			'name'=> $sberIdLogin,
			'mail'=> $user_data['email'],
			'pass'=> $this->generateRandomString(10),
			'status' => 1
			));
		$userCreation->save();
		$userInfo = user_load_by_name($sberIdLogin);
		if($userInfo) {
			return $userInfo;
		}

 		return false;
	}

	private function sber_id_user_find_by_sub($sub) {
		$connection = \Drupal::service('database');
		$query = $connection->query("SELECT * FROM {sber_id_users} WHERE sub = :sub", [':sub' => $sub]);
		$result = $query->fetchAll();
		if($result) {
			return $result[0];
		}
		return false;

	}
	
	private function drupal_user_find_by_id($uid) {
		$account = \Drupal\user\Entity\User::load($uid);
		if($account) {
			return $account;
		}
		return false;
	}

	private function drupal_user_find_by_email($email) {
		$userInfo = user_load_by_mail($email);
		if($userInfo) {
			return $userInfo;
		}
		return false;
	}

	private function finish_auth($userInfo) {
		$this->oauth_handler->reqComplete();
		user_login_finalize($userInfo);
	}
	private function generateRandomString($length = 32) {
	    $characters = 'ABCDEFabcdef0123456789';
	    $charactersLength = strlen($characters);
	    $randomString = '';
	    for ($i = 0; $i < $length; $i++) {
	        $randomString .= $characters[rand(0, $charactersLength - 1)];
	    }
	    return $randomString;
	}
	private function show_login_form($userInfo,$drupal_user,$auth_error = false) {
	    $config = \Drupal::config('system.site');

	    // Создаем объект CacheableMetadata и устанавливаем max-age в 0.
	    $cache_metadata = new CacheableMetadata();
	    $cache_metadata->setCacheMaxAge(0);


	    

		$output = array();
	    $output['#theme'] = 'auth_form';
	    $output['#sber_id_params'] = array(
	    	'username' => $drupal_user ? $drupal_user->getUsername() : '',
	    	'sitename' => $config->get('name'),
	    	'auth_error' => $auth_error,
	    	'sberid_user' => [
	    		'phone' => $userInfo['phone'],
	    		'email' => $userInfo['email'],
				'sub' => $userInfo['sub'],
				'aud' => $userInfo['aud'],
				'family_name' => $userInfo['family_name'],
				'given_name' => $userInfo['given_name'],
				'middle_name' => $userInfo['middle_name'],
	    	]
	    );
	    $output['#sber_id_params']['need_email'] = strlen($userInfo['email']) === 0 ? true : false;

	   	// Связываем объект CacheableMetadata с данными.
	    $cache_metadata->applyTo($output);
	    
	    return $output;
	}

}
