<?php

namespace Drupal\sber_id\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\sber_id\Plugin\Network\SberIdHandler;

/**
 * Settings form for Social API Twitter.
 */
class SberidSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sberid_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'sber_id.settings',
    ];
  }

  private function CheckSertificateStatus($mode = 'TEST') {

    $module_path = DRUPAL_ROOT . '/' . drupal_get_path('module', 'sber_id');
    $sert_dir =  $module_path . '/sert/';
    $log_dir =  $module_path . '/log/';

    $sber_id_handler = new SberIdHandler('v5', $mode);
    $sber_id_handler->setSertificateDir($sert_dir);
    $sber_id_handler->setLogDir($log_dir);

    $sertificate_file_exist_status = $sber_id_handler->sertificate_check_exist();
    $sertificate_file_decoded_status = $sber_id_handler->sertificate_check_decoded();

    if($sertificate_file_decoded_status) {
      $sertificate_upload_status = 'Статус: Сертификат успешно загружен и расшифрован.';
      $sertificate_upload_style = 'green';
    } else if($sertificate_file_exist_status) {
      $sertificate_upload_style = '#e19713';
      $sertificate_upload_status = 'Статус: Сертификат успешно загружен. Укажите корректный пароль для расшифровки.';
    } else {
      $sertificate_upload_style = 'red';
      $sertificate_upload_status = 'Статус: Файл сертификата не обнаружен.';
    }
    return [
      'upload_style' => $sertificate_upload_style,
      'upload_status' => $sertificate_upload_status,
    ];
  }

  private function saveSertificate($mode = 'TEST',$post_password,$file_name) {
    $config = $this->config('sber_id.settings');
    $module_path = DRUPAL_ROOT . '/' . drupal_get_path('module', 'sber_id');
    $sert_dir =  $module_path . '/sert/';
    $log_dir =  $module_path . '/log/';

    $sber_id_handler =  new SberIdHandler('v5', $mode);

    $sber_id_handler->setSertificateDir($sert_dir);
    $sber_id_handler->setLogDir($log_dir);
    $sert_password = $post_password;
    $file = [
      'name' => $_FILES['files']['name'][$file_name],
      'tmp_name' =>$_FILES['files']['tmp_name'][$file_name],
    ];
    if(strlen($file['name']) > 1) {
      // echo "<pre>";
      // print_r($sert_password);
      // die;
      $sber_id_handler->sertificate_save($file);
      if(!$sber_id_handler->sertificate_crop($sert_password)) {
        // $sert_password = '';
      }
    } else if($sber_id_handler->sertificate_check_exist() && !$sber_id_handler->sertificate_check_decoded()) {
      if(!$sber_id_handler->sertificate_crop($sert_password)) {
        $sert_password = '';
      }
    } else if($sber_id_handler->sertificate_check_decoded() && $sert_password != $config->get('sertificate_password') ) {
      if(!$sber_id_handler->sertificate_crop($sert_password)) {
        $sert_password = $mode == 'PROM' ? $config->get('sertificate_password') : $config->get('test_sertificate_password');;
      }
    }
    return $sert_password;
  }
  /**
   * Build Admin Settings Form.
   */


  public function buildForm(array $form, FormStateInterface $form_state) {
   
    $config = $this->config('sber_id.settings');

    $prom_sertificate_status = $this->CheckSertificateStatus('PROM');
    $test_sertificate_status = $this->CheckSertificateStatus('TEST');

    $form = [
      '#attributes' => ['enctype' => "multipart/form-data"],
    ];

    $form['sber_id_base_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Основные настройки'),
      '#open' => TRUE,
    ];

    $form['sber_id_base_settings']['api_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Режим API'),
      '#required' => true,
      '#options' => array(
         'PROM' => $this->t('Сбер Id Prom'),
         'TEST' => $this->t('СБер Id Test'),
       ),
      '#default_value' => $config->get('api_mode') ? $config->get('api_mode') : 'TEST',
      '#description' => $this->t(''),
    ];

    // PROM SETTINGS

    $form['sber_id_prod_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Данные для подключения к Prom API'),
      '#open' => TRUE,
    ];
    $form['sber_id_prod_settings']['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#required' => TRUE,
      '#default_value' => $config->get('client_id'),
      '#description' => $this->t('Укажите client_id '),
    ];
    $form['sber_id_prod_settings']['client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Secret'),
      '#required' => TRUE,
      '#default_value' => $config->get('client_secret'),
      '#description' => $this->t('Укажите client_secret '),
    ];
    $form['sber_id_prod_settings']['sertificate_file_loaded'] = [
      '#type' => 'label',
      '#title' => $this->t($prom_sertificate_status['upload_status']),
      '#attributes' => array(
        'style'=>'font-weight:normal;color:' . $prom_sertificate_status['upload_style']
      ),
    ];
    $form['sber_id_prod_settings']['sertificate_file'] = [
      '#type' => 'file',
      '#title' => $this->t('Файл сертификата'),
      '#required' => FALSE,
      '#default_value' => '',

      '#description' => $this->t('Загрузите файл сертификата'),
    ];
    $form['sber_id_prod_settings']['sertificate_password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Пароль от сертификата'),
      '#required' => FALSE,
      '#default_value' => $config->get('sertificate_password'),
      '#description' => $this->t('Укажите пароль от файла сертификата'),
    ];



    // TEST SETTINGS

    $form['sber_id_test_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Данные для подключения к Test API'),
      '#open' => true,
    ];
    $form['sber_id_test_settings']['test_client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#required' => FALSE,
      '#default_value' => $config->get('test_client_id'),
      '#description' => $this->t('Укажите client_id '),
    ];
    $form['sber_id_test_settings']['test_client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client Secret'),
      '#required' => FALSE,
      '#default_value' => $config->get('test_client_secret'),
      '#description' => $this->t('Укажите client_secret '),
    ];
    $form['sber_id_test_settings']['test_sertificate_file_loaded'] = [
      '#type' => 'label',
      '#title' => $this->t($test_sertificate_status['upload_status']),
      '#attributes' => array(
        'style'=>'font-weight:normal;color:' . $test_sertificate_status['upload_style']
      ),
    ];
    $form['sber_id_test_settings']['test_sertificate_file'] = [
      '#type' => 'file',
      '#title' => $this->t('Файл сертификата'),
      '#required' => FALSE,
      '#default_value' => '',
      '#description' => $this->t('Загрузите файл сертификата'),
    ];
    $form['sber_id_test_settings']['test_sertificate_password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Пароль от сертификата'),
      '#required' => FALSE,
      '#default_value' => $config->get('test_sertificate_password'),
      '#description' => $this->t('Укажите пароль от файла сертификата'),
    ];


    // OTHER SETTINGS

    $form['sber_id_personal_settings']['personal_button_description'] = [
      '#type' => 'label',
      '#title' => '«Данная настройка позволяет включить персонализированную кнопку, она даёт возможность сделать вход для пользователя более привычным и, в среднем, увеличивает конверсию входа в полтора раза. 
                    Быстрый вход позволит клиентам, если они ранее давали согласие для вашего сервиса, входить по одному клику. Для включения персонализированной кнопки
                     и быстрого входа Вам нужно добавить домен в список доверенных, напишите нам письмо на почту support@ecom.sberbank.ru и мы сделаем нужные настройки»',
      '#attributes' => array(
        'style'=>'max-width:800px;'
      ),
    ];
    $form['sber_id_personal_settings']['personal_button'] = [
      '#type' => 'checkbox',
      '#title' => 'Персонализированная кнопка',
      '#default_value' => $config->get('personal_button') ? $config->get('personal_button') : false,
      '#description' => $this->t(''),
    ];
    $form['sber_id_personal_settings']['fast_login'] = [
      '#type' => 'checkbox',
      '#title' => 'Быстрый вход',
      '#default_value' => $config->get('fast_login') ? $config->get('fast_login') : false,
      '#description' => $this->t(''),
    ];


    $form['sber_id_other_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Дополнительные настройки'),
      '#open' => TRUE,
    ];
    // $form['sber_id_other_settings']['api_version'] = [
    //   '#type' => 'select',
    //   '#title' => $this->t('Версия API'),
    //   '#required' => FALSE,
    //   '#options' => array(
    //      '2.1.0' => $this->t('2.1.0'),
    //      '3.0.0' => $this->t('3.0.0'),
    //    ),
    //   '#default_value' => $config->get('api_version') ? $config->get('api_version') : '3.0.0',
    //   '#description' => $this->t(''),
    // ];
    // $form['sber_id_other_settings']['use_pkce'] = [
    //   '#type' => 'checkbox',
    //   '#title' => $this->t('PKCE'),
    //   '#required' => FALSE,
    //   '#default_value' => $config->get('use_pkce'),
    //   '#description' => $this->t('включить режим use_pkce'),
    // ];
    // $form['sber_id_other_settings']['use_mweb2app'] = [
    //   '#type' => 'checkbox',
    //   '#title' => $this->t('mweb2app'),
    //   '#required' => FALSE,
    //   '#default_value' => $config->get('use_mweb2app'),
    //   '#description' => $this->t('Включить режим mWeb2app '),
    // ];
    $form['sber_id_other_settings']['scopes'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Поля (Scopes)'),
      '#required' => FALSE,
      '#options' => array(
         'name' => $this->t('ФИО'),
         'email' => $this->t('E-mail'),
         'mobile' => $this->t('Телефон'),
         'birthdate' => $this->t('Дата рождения'),
         'gender' => $this->t('Пол'),
         'address_of_actual_residence' => $this->t('Адрес места жительства'),
       ),
      '#default_value' => array_merge(array('name','email'), $config->get('scopes')),
      '#description' => $this->t('Выберите поля которые будет запрашивать модуль при авторизации в Сбер ID'),
    ];

    $form['sber_id_button_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Настройки отображения кнопки'),
      '#open' => TRUE,
    ];
    $form['sber_id_button_settings']['button_size'] = [
      '#type' => 'select',
      '#title' => $this->t('Размер кнопки'),
      '#required' => FALSE,
      '#options' => array(
         'SMALL' => $this->t('Маленькая'),
         'MEDIUM' => $this->t('Средняя'),
         'LARGE' => $this->t('Большая'),
       ),
      '#default_value' => $config->get('button_size') ? $config->get('button_size') : 'MEDIUM',
      '#description' => $this->t(''),
    ];
    $form['sber_id_button_settings']['button_form'] = [
      '#type' => 'select',
      '#title' => $this->t('Форма кнопки'),
      '#required' => FALSE,
      '#options' => array(
         'BOX' => $this->t('Прямоугольная'),
         'CIRCLE' => $this->t('Круглая'),
       ),
      '#default_value' => $config->get('button_form') ? $config->get('button_form') : 'BOX',
      '#description' => $this->t(''),
    ];
    $form['sber_id_button_settings']['button_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Тема'),
      '#required' => FALSE,
      '#options' => array(
         'FLAT' => $this->t('Зеленая'),
         'BORDER' => $this->t('Белая'),
       ),
      '#default_value' => $config->get('button_style') ? $config->get('button_style') : 'GREEN',
      '#description' => $this->t(''),
    ];

    $form['sber_id_notification_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Настройки персонализированного баннера'),
      '#open' => TRUE,
    ];
    $form['sber_id_notification_settings']['notification_position'] = [
      '#type' => 'select',
      '#title' => $this->t('Тема'),
      '#required' => FALSE,
      '#options' => array(
         'white' => $this->t('Белая'),
         'dark' => $this->t('Темная'),
       ),
      '#default_value' => $config->get('notification_position') ? $config->get('notification_position') : 'white',
      '#description' => $this->t(''),
    ];
    $form['sber_id_notification_settings']['notification_theme'] = [
      '#type' => 'select',
      '#title' => $this->t('Расположение'),
      '#required' => FALSE,
      '#options' => array(
         'bottom-right' => $this->t('Снизу-справа'),
         'bottom-left' => $this->t('Снизу-слева'),
         'top-right' => $this->t('Вверху-справа'),
         'top-left' => $this->t('Вверху-слева'),
       ),
      '#default_value' => $config->get('notification_theme') ? $config->get('notification_theme') : 'bottom-right',
      '#description' => $this->t(''),
    ];



    return parent::buildForm($form, $form_state);
  }

  /**
   * Submit Common Admin Settings.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    
    $values = $form_state->getValues();
    
    $prom_sert_password = $this->saveSertificate('PROM', $values['sertificate_password'],'sertificate_file');
    $test_sert_password = $this->saveSertificate('TEST', $values['test_sertificate_password'],'test_sertificate_file');

    $this->config('sber_id.settings')
      ->set('api_mode', $values['api_mode'])
      ->set('client_id', $values['client_id'])
      ->set('client_secret', $values['client_secret'])
      ->set('sertificate_password', $prom_sert_password)
      ->set('test_client_id', $values['test_client_id'])
      ->set('test_client_secret', $values['test_client_secret'])
      ->set('test_sertificate_password', $test_sert_password)
      ->set('use_pkce', $values['use_pkce'])
      // ->set('use_mweb2app', $values['use_mweb2app'])
      ->set('scopes', $values['scopes'])
      ->set('button_size', $values['button_size'])
      ->set('button_form', $values['button_form'])
      ->set('button_style', $values['button_style'])
      ->set('api_version', '5.0.0')
      ->set('personal_button', $values['personal_button'])
      ->set('fast_login', $values['fast_login'])
      ->set('notification', true)
      ->set('notification_position', $values['notification_position'])
      ->set('notification_theme', $values['notification_theme'])
      ->save();

    drupal_set_message($this->t('Настройки успешно обновлены'));
  }

}
    // echo "<pre>";
    // print_r($values);
// die;