<?php

namespace Drupal\sber_id\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\user\Entity\User;
use Drupal\sber_id\Plugin\Network\SberIdHandler;

/**
 * Provides a Sber Id block
 *
 * @Block(
 *   id = "sberid_block",
 *   admin_label = @Translation("Сбер ID"),
 *   category = @Translation("Кнопка авторизации")
 * )
 */
class SberidBlock extends BlockBase {
  /**
   * {@inheritdoc}
   */
  public function build() {
    
    $moduleConfig = \Drupal::config('sber_id.settings');


    $sber_id_handler = new SberIdHandler($moduleConfig->get('api_version'),$moduleConfig->get('api_mode'));
    $path = drupal_get_path('module', 'sber_id');

    if($moduleConfig->get('api_mode') === 'TEST') {
      $sber_id_handler->setClientId($moduleConfig->get('test_client_id'));
      $sber_id_handler->setClientSecret($moduleConfig->get('test_client_secret'));
    } else {
      $sber_id_handler->setClientId($moduleConfig->get('client_id'));
      $sber_id_handler->setClientSecret($moduleConfig->get('client_secret'));
    }
    $sber_id_handler->setProperty('pkce',$moduleConfig->get('pkce') ? true : false);


    $auth_url = $sber_id_handler->generate_oauth_url();
    $auth_params = $sber_id_handler->get_oauth_params();


    $sber_id_params = array(
      'client_id' => $moduleConfig->get('client_id'),
      'client_secret' => $moduleConfig->get('client_secret'),
      'web_to_app' => $moduleConfig->get('use_mweb2app'),
      'pkce' => $moduleConfig->get('pkce'),
      'scopes' => str_replace('+', ' ', $moduleConfig->get('scopes')),
      'personal' => $moduleConfig->get('personal_button'),
      'fast' => $moduleConfig->get('fast_login'),
      'button_size' => $moduleConfig->get('button_size'),
      'button_form' => $moduleConfig->get('button_form'),
      'button_style' => $moduleConfig->get('button_style'),
      'notification' => $moduleConfig->get('notification'),
      'notification_theme' => $moduleConfig->get('notification_theme'),
      'notification_position' => $moduleConfig->get('notification_position'),

      'auth_params' => $auth_params,
      'base_url' => $sber_id_handler->getBaseUrl(),
    );

    $sber_id_params['sber_id']['button_type'] = 'default';
    $sber_id_params['sber_id']['button_size'] = 'default';
    $sber_id_params['sber_id']['button_theme'] = 'default';

    if($moduleConfig->get('button_size') == 'SMALL') {
       $sber_id_params['sber_id']['button_size'] = 'small';
    } else if($moduleConfig->get('button_size') == 'LARGE') {
       $sber_id_params['sber_id']['button_size'] = 'long';
    }

    if($moduleConfig->get('button_style') == 'BORDER') {
     $sber_id_params['sber_id']['button_theme'] = 'white';
    }
    
    if (\Drupal::currentUser()->isAuthenticated()) {
      return array(
        '#cache' => [
          'max-age' => 0,
        ]
      );
    }

    return array(
      '#theme' => 'sberid_button',
      '#test_var' => 'test variable',
      '#sber_id' => $sber_id_params,
      '#cache' => [
        'max-age' => 0,
      ],
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

}